import argparse
from .fnr_core import load_keywords_to_be_replaced, find_and_replace

def parse_cla():
    parser = argparse.ArgumentParser()
    parser.add_argument("patient", help="Filename - The original input file containing the content that we would like to find and replace", type=str)
    parser.add_argument('-f', '--context-file', help="Filename - An input file containing set of key value pairs to find & replace. The placeholder keywords you're looking to replace should have curly braces {} around it", type=str)
    parser.add_argument("--in-place", help="Use if you would not like a new file to be generated and you want to just replace the text in the original file", action="store_true")

    return parser.parse_args()

def main():
    print('Hello, fnr!')
    args = parse_cla()

    # Get the mapping of items to be replaced
    context = {}
    try:
        context = load_keywords_to_be_replaced(args)
    except:
        print('Failed to load keyword mapping of values to be replaced')
        print('Exiting now...')
        exit()

    # Read in
    template = ''
    with open(args.patient, 'r') as in_file:
        template = in_file.read()

    # Write out
    out_file = (args.patient if args.in_place else 'newfile.out')
    with open(out_file, 'w') as out:
        out.write(find_and_replace(context, template))

if __name__ == '__main__':
    main()



from abc import ABC, abstractmethod

class context_loader(ABC):
    def __init__(self):
        super().__init__()

    @abstractmethod
    def load_context_vars(self):
        pass

class file_loader(context_loader):
    def __init__(self, f):
        self.file = f
        self.file_format = 'csv'

    def __file_not_found_handler(self):
        print('Err: Input file ({}) not found'.format(self.file) )

    def show(self):
        try:
            with open(self.file) as context_file:
                print(context_file.read())
        except FileNotFoundError:
            self.__file_not_found_handler()

    def load_context_vars(self):
        try:
            with open(self.file) as context_file:
                return {row.split(',')[0]:row.strip().split(',')[1] for row in context_file}
        except FileNotFoundError:
            self.__file_not_found_handler()

class console_loader(context_loader):
    def __init__(self):
        self.context = {}

    def __load_new_pair(self):
        print('---------------------')
        find = input('Enter value to find: ')

        if find == '':
            return -1

        replace = input('Enter value for replacement: ')

        self.context[find] = replace
        print('---------------------')

    def load_context_vars(self):
        self.__load_new_pair()
        print('You may proceed to add more find & replace keys below. Use an empty find value to end (press Enter).')

        while(self.__load_new_pair() != -1):
            pass

        return self.context


from setuptools import setup

with open("README.md", 'r') as ld:
    long_description = ld.read()

setup(
        name = 'fnr',
        version = '0.1.2',
        packages = ['fnr'],
        author = 'Collin Choy',
        author_email = 'collinchoyccc@gmail.com',
        description = 'A command line utility to find & replace text.',
        long_description=long_description,
        url='https://github.com/collincchoy/fnr',
        classifiers=[
            'Programming Language :: Python :: 3'
        ],
        test_suite = 'fnr.tests.fnrTests',
        setup_requires = ['nose>=1.0'],
        entry_points = {
            'console_scripts': [
                'fnr = fnr.fnr_cli:main'
            ]
        }
)



from .context_loaders import file_loader, console_loader
import re

def load_keywords_to_be_replaced(args):
    if (args.context_file):
        print('Loading the context variables from {} now...'.format(args.context_file))
        context_file = file_loader(args.context_file)
    else:
        context_file = console_loader()

    return context_file.load_context_vars()

def find_and_replace(context, template):
    # escape special regex characters from keys
    context = {re.escape(k):v for k,v in context.items()}
    pattern = re.compile('|'.join(context.keys()))
    return re.sub(pattern, lambda match: context[re.escape(match.group())], template)